unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, VCLTee.TeEngine, VCLTee.TeCanvas,
  VCLTee.Series, Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.TeeEdit;

type
  TForm1 = class(TForm)
    Image1: TImage;
    ChartEditor1: TChartEditor;
    Chart1: TChart;
    procedure Image1Click(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateChart;
    function CreateChart: TChart;
    procedure GapChartAfterDraw(Sender: TObject);
    procedure GapChartLegendDrawSymbol(Sender: TObject; Series: TChartSeries;
      ValueIndex: Integer; R: TRect);
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses Math;

const
  SCALE_FACTOR = 4;

procedure TForm1.GapChartAfterDraw(Sender: TObject);
begin

end;

procedure TForm1.GapChartLegendDrawSymbol(Sender: TObject; Series: TChartSeries;
  ValueIndex: Integer; R: TRect);
const
  COLORS: array[0..0] of TColor = (clGreen);
var
  I: Integer;
  lCanvas: TCanvas3D;
  lLeft: Integer;
  lNumColors: Integer;
  lRight: Integer;
  lWidth: Integer;
  lR: TRect;
begin
  if Length(COLORS) > 0 then
  begin
    lNumColors := 3;

    lCanvas := Series.ParentChart.Canvas;

    lLeft := R.Left;
    lWidth := R.Width div lNumColors;

    for I := 0 to 2 do
    begin
      lCanvas.Pen.Color := COLORS[I];
      lCanvas.Brush.Color := COLORS[I];

      if I = (lNumColors-1) then
        lRight := R.Right
      else
        lRight := lLeft + lWidth;

      lR := Rect(lLeft, R.Top, lRight, R.Bottom);
      lCanvas.FillRect(lR);

      lLeft := lRight;
    end;
  end;
end;

procedure TForm1.Image1Click(Sender: TObject);
begin
  UpdateChart;

end;

constructor TForm1.Create(aOwner: TComponent);
begin
  inherited;
  UpdateChart;
end;

function TForm1.CreateChart: TChart;
begin
  result := TChart.Create(nil);

  //result.Canvas := TTeeCanvas3D.Create;  doesn't allow transparent series for batch gap chart

  result.Color := clWhite;
  result.BevelOuter := bvNone;
  result.LeftAxis.Grid.Color := clGray;
  result.LeftAxis.Grid.Visible := True;//FGapReportLayout.ChartOptions.ShowHorzLines;
  result.AxisBehind := False;//not FGapReportLayout.ChartOptions.ShowLinesInFront;

  result.LeftAxis.LabelsFont.Name := 'Arial Narrow'; {added from Gap Chart}
  result.LeftAxis.AutomaticMinimum := False; {added from Gap Chart}
  result.LeftAxis.Minimum := 0; {added from Gap Chart}
  result.LeftAxis.MinorTicks.Visible := False; {added from Gap Chart}
  result.LeftAxis.TicksInner.Visible := False; {added from Gap Chart}
  result.LeftAxis.Ticks.Style := psDot;
  result.LeftAxis.TickLength := 2; {added from Gap Chart}
  result.LeftAxis.Axis.Width := 1;
  result.LeftAxis.Title.Font.Name := 'Arial Narrow';

  result.BottomAxis.Axis.Color := clSilver;
  result.BottomAxis.MinorTicks.Visible := false; {added from Gap Chart}
  result.BottomAxis.Ticks.Visible := false;
  result.BottomAxis.LabelsFont.Name := 'Arial Narrow';
  result.BottomAxis.Grid.Visible := False;
  result.BottomAxis.Axis.Width := 1;

  result.Legend.Frame.Visible := False;
  result.Legend.Title.Font.Name := 'Arial Narrow';
  result.Legend.Title.Font.Style := [];
  result.Legend.LegendStyle := lsSeries;
  result.Legend.Shadow.Visible := false;
  result.Legend.Alignment := laLeft;
  result.Legend.Transparent := True;
  result.Legend.Symbol.Squared := True;
  result.Legend.Symbol.Width := result.Legend.Symbol.Height;
  result.Legend.Symbol.Shadow.Visible := false;

  result.LeftAxis.Axis.Color := clSilver;
  result.LeftAxis.AxisValuesFormat := '$,0';

  result.Walls.Visible := False;

  result.BackWall.Pen.Color := clSilver;
//  result.OnGetAxisLabel := ChartAxisLabel;

  result.View3D := False;
end;

procedure AssignChartToGraphic(aChart: TChart; const aRect: TRect; aGraphic: TGraphic; aTransparent: boolean = false);
var
  lHoldShadow: boolean;
  I: Integer;
  lBoolArray: array of boolean;
  lHoldBevel: TBevelCut;
  lHoldColor: TColor;
  lHoldGradientVisible: boolean;
  lMetafile: TMetafile;
  {$ifopt C+}
  lCanvasClassName: string;
  {$endif}
  lBmp: TBitmap;
  lCanvasClass: TCanvas3DClass;
  lHoldHover: Boolean;
begin
  {$ifopt C+}
  lCanvasClassName := aChart.Canvas.ClassName;
  {$endif}
  lCanvasClass := TCanvas3DClass(aChart.Canvas.ClassType);
  if lCanvasClass <> TTeeCanvas3D then
    aChart.Canvas := TTeeCanvas3D.Create;

  lHoldGradientVisible := aChart.Gradient.Visible;
  lHoldColor := aChart.Color;
  lHoldBevel := aChart.BevelOuter;
  lHoldShadow := aChart.Shadow.Visible;
  lHoldHover := aChart.Hover.Visible;

  try
    SetLength(lBoolArray, aChart.SeriesCount);
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
      begin
        lBoolArray[I] := TPieSeries(aChart.Series[I]).Circled;
        TPieSeries(aChart.Series[I]).Circled := True;
      end
      else
        lBoolArray[I] := True;


    aChart.Gradient.Visible  := false;
    if not aTransparent then
      aChart.Color := clWhite
    else
      aChart.Color := VCL.Graphics.clNone; //clWhite;
    aChart.BevelOuter := bvNone;
    aChart.BevelInner := bvNone;
    aChart.Shadow.Visible := False;
    aChart.Hover.Visible := false;

    if aGraphic is TMetafile then
    begin
      lMetafile := aChart.TeeCreateMetafile(false, aRect);
      try
        aGraphic.Assign(lMetafile);
      finally
        lMetafile.Free;
      end;
    end
    else
    begin
      lBmp := aChart.TeeCreateBitmap(clWhite, aRect);
      try
        aGraphic.Assign(lBmp);
      finally
        lBmp.Free;
      end;
    end;
  finally
    aChart.Hover.Visible := lHoldHover;
    aChart.Shadow.Visible := lHoldShadow;
    aChart.Gradient.Visible := lHoldGradientVisible;
    aChart.Color := lHoldColor;
    aChart.BevelOuter := lHoldBevel;
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
        TPieSeries(aChart.Series[I]).Circled := lBoolArray[I];
    if (lCanvasClass <> aChart.Canvas.ClassType) then
      aChart.Canvas := lCanvasClass.Create;
  end;

  {$ifopt C+}
  assert(SameText(lCanvasClassName, aChart.Canvas.ClassName),
    Format('Chart canvas should be %s but is %s', [lCanvasClassName, aChart.Canvas.ClassName]));
  {$endif}
end;


procedure TForm1.UpdateChart;
var
  lChart: TChart;
  lVersion: Integer;
  I: Integer;
  lsrs: TAreaSeries;
  lRect: TRect;
  lBmp: TBitmap;
begin
  lChart := CreateChart;
  try
    lChart.LeftAxis.Title.Font.Size := 10;

    lChart.BottomAxis.Title.Caption := 'Years';
    lChart.BottomAxis.Title.Font.Size := 10;
    lChart.Legend.Alignment := laRight;
    lChart.Legend.VertSpacing := 40;
    lChart.Legend.Symbol.Squared := True;

    for I := 1 to 3 do
    begin
      lsrs := TAreaSeries.Create(self);
      lsrs.MultiArea := maStacked;
      lsrs.ParentChart := lChart;
      lsrs.FillSampleValues(10);
    end;

    lChart.Legend.Font.Size := Ceil(lChart.Legend.Font.Size * SCALE_FACTOR);
    lChart.BottomAxis.LabelsFont.Size := Ceil(lChart.BottomAxis.LabelsFont.Size * SCALE_FACTOR);
    lChart.BottomAxis.Title.Font.Size := Ceil(lChart.BottomAxis.Title.Font.Size * SCALE_FACTOR);
    lChart.LeftAxis.LabelsFont.Size := Ceil(lChart.LeftAxis.LabelsFont.Size * SCALE_FACTOR);
    lChart.LeftAxis.Title.Font.Size := Ceil(lChart.LeftAxis.Title.Font.Size * SCALE_FACTOR);


    lRect := Image1.ClientRect;
    lRect.Right := Ceil(lRect.Right * SCALE_FACTOR);
    lRect.Bottom := Ceil(lRect.Bottom * SCALE_FACTOR);

    lChart.SetBounds(0, 0, lRect.Width, lRect.Height);

    lChart.Draw;
    lChart.Draw;

    lBmp := TBitmap.Create;

    lBmp.Width := lRect.Right;
    lBmp.Height := lRect.Bottom;

    lBmp := TBitmap.Create;
    try
      lBmp.SetSize(lRect.Right, lRect.Height);
      AssignChartToGraphic(lChart, lRect, lBmp);
      Image1.Picture.Graphic := lBmp;
    finally
      lBmp.Free;
    end;
  finally // wrap up
    lChart.FreeAllSeries;
    lChart.Free;
  end;    // try/finally
end;

end.
